@@ -39,7 +39,9 @@ class gPodderExtension:
         basename, ext = os.path.splitext(filename)
 
         new_basename = util.sanitize_encoding(title) + ext
-        new_basename = util.sanitize_filename(new_basename)
+        # On Windows, force ASCII encoding for filenames (bug 1724)
+        new_basename = util.sanitize_filename(new_basename,
+                use_ascii=gpodder.ui.win32)
         new_filename = os.path.join(dirname, new_basename)
 
         if new_filename == current_filename:
